// Copyright (c) 2015 Uber Technologies, Inc.

// Permission is hereby granted, free of charge, to any person obtaining a copy
// of this software and associated documentation files (the "Software"), to deal
// in the Software without restriction, including without limitation the rights
// to use, copy, modify, merge, publish, distribute, sublicense, and/or sell
// copies of the Software, and to permit persons to whom the Software is
// furnished to do so, subject to the following conditions:
//
// The above copyright notice and this permission notice shall be included in
// all copies or substantial portions of the Software.
//
// THE SOFTWARE IS PROVIDED "AS IS", WITHOUT WARRANTY OF ANY KIND, EXPRESS OR
// IMPLIED, INCLUDING BUT NOT LIMITED TO THE WARRANTIES OF MERCHANTABILITY,
// FITNESS FOR A PARTICULAR PURPOSE AND NONINFRINGEMENT. IN NO EVENT SHALL THE
// AUTHORS OR COPYRIGHT HOLDERS BE LIABLE FOR ANY CLAIM, DAMAGES OR OTHER
// LIABILITY, WHETHER IN AN ACTION OF CONTRACT, TORT OR OTHERWISE, ARISING FROM,
// OUT OF OR IN CONNECTION WITH THE SOFTWARE OR THE USE OR OTHER DEALINGS IN
// THE SOFTWARE.

package hyperbahn

import (
	"net"
	"strconv"

	"github.com/uber/tchannel-go/hyperbahn/gen-go/hyperbahn"
)

// intToIP4 converts an integer IP representation into a 4-byte net.IP struct
func intToIP4(ip uint32) net.IP {
	return net.IP{
		byte(ip >> 24 & 0xff),
		byte(ip >> 16 & 0xff),
		byte(ip >> 8 & 0xff),
		byte(ip & 0xff),
	}
}

// servicePeerToHostPort converts a Hyperbahn ServicePeer into a hostPort string.
func servicePeerToHostPort(peer *hyperbahn.ServicePeer) string {
	host := intToIP4(uint32(*peer.IP.Ipv4)).String()
	port := strconv.Itoa(int(peer.Port))
	return net.JoinHostPort(host, port)
}
